#include "webpage.h"

//Einbinden der Bibliotheken
//für die WiFi Verbindung des ESP32
#include <WiFi.h>
#include <WebServer.h>
#include <HTTPClient.h>

#include <ArduinoJson.h>
#include <HTTPClient.h>
#include <WiFiMulti.h>

//SSID & Passwort für das lokale WiFi
#define WIFI_SSID ""
#define WIFI_PASSWORD ""

WebServer server(80);
WiFiClient client;

HTTPClient https;

WiFiMulti wifiMulti;

String chatgptApiKey = "****";
String chatgptServer = "https://api.openai.com/v1/completions";

void setup() {
  //begin der seriellen Kommunikation mit 115200 baud
  Serial.begin(115200);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  int maxConnectionAttempts = 10;
  int counter = 0;
  while (WiFi.status() != WL_CONNECTED && counter < maxConnectionAttempts) {
    Serial.print(".");
    counter++;
    delay(200);
  }

  //Wenn die Verbindung erfolgreich aufgebaut wurde,
  //dann soll die IP-Adresse auf der seriellen Schnittstelle
  //ausgegeben werden.
  Serial.println("");
  Serial.print("Zum WiFi Netzwerk ");
  Serial.print(WIFI_SSID);
  Serial.println(WiFi.isConnected() ? " verbunden" : "nicht verbunden");

  if (WiFi.isConnected()) {
    Serial.print("IP-Adresse: ");
    Serial.println(WiFi.localIP());
    Serial.println();

    server.begin();
    server.on("/", callWebsite);
  } else {
    Serial.print("Fehler beim Aufbau der Verbindung zu ");
    Serial.println(WIFI_SSID);
    Serial.println();
  }
}

void callWebsite() {
  String chatgptQuestion = "";
  for (int i = 0; i < server.args(); i++) {
    String parameterName = server.argName(i);
    String parameterValue = server.arg(i);
    if (parameterName == "inputMsg") {
      chatgptQuestion = parameterValue;
    }
  }

  String htmlpage = page;
  htmlpage.replace("%inputFieldValue%", chatgptQuestion);
  String errorMsg = "";

  if (chatgptQuestion.length() > 0 && https.begin(chatgptServer)) {
    Serial.print("request: ");
    Serial.println(chatgptQuestion);
    //definieren das wir Daten vom Typ "application/json" senden
    https.addHeader("Content-Type", "application/json");
    //den API Key mit
    String token_key = "Bearer 1" + chatgptApiKey;
    https.addHeader("Authorization", token_key);

    StaticJsonDocument<200> data;
    //das verwendete Model
    data["model"] = "text-davinci-003";
    //die Anfrage an das System
    data["prompt"] = chatgptQuestion;
    //die Komplexität
    data["temperature"] = 0;
    //die gewünschte länge der Antwort von ChatGPT
    //Dieser Wert darf nicht zu groß gewählt werden,
    //der ESP32 hat nur begrenzten Speicherplatz.
    data["max_tokens"] = 300;
    String requestBody;
    //serialisieren des JSON in einen String
    serializeJson(data, requestBody);
    //absenden des Requests
    int httpCode = https.POST(requestBody);
    if (httpCode != 200) {
      errorMsg = "HTTP-Code: " + String(httpCode);

      switch (httpCode) {
        case 401: errorMsg += " Unauthorized"; break;
        case 403: errorMsg += " Forbidden"; break;
        case 404: errorMsg += " Not Found"; break;
        case 405: errorMsg += " Method Not Allowed"; break;
        case 408: errorMsg += " Timeout Error"; break;
        case 500: errorMsg += " Internal Server Error"; break;
        case 501: errorMsg += " Not Implemented"; break;
        case 502: errorMsg += " Bad Gateway"; break;
        case 504: errorMsg += " Gateway Timeout"; break;
        default: errorMsg += "";
      }
    }

    String response = https.getString();
    if (response.length() > 0) {
      StaticJsonDocument<1024> doc;
      DeserializationError error = deserializeJson(doc, response);
      if (error) {
        errorMsg = error.f_str();
        Serial.print("error: ");
        Serial.println(errorMsg);
        return;
      }

      if (httpCode > 200) {
        errorMsg += "<br/>";
        String errorMessage = doc["error"]["message"];
        errorMsg += errorMessage;
        htmlpage.replace("%resultTxt%", "");
      } else {
        String chatGPTAnswer = doc["choices"][0]["text"];
        Serial.print("answer: ");
        Serial.println(chatGPTAnswer);
        htmlpage.replace("%resultTxt%", chatGPTAnswer);
      }
    }
  } else {
    htmlpage.replace("%resultTxt%", "");
  }

  htmlpage.replace("%errorMsg%", errorMsg);
  String errorMsgStyle = " style='display:none;' ";
  if (errorMsg.length() > 0) {
    errorMsgStyle = " style='display:block;' ";
  }

  htmlpage.replace("%errorMsgStyle%", errorMsgStyle);

  server.send(200, "text/html", htmlpage);
}

/**
* Funktion wird fortlaufen ausgeführt.
**/
void loop() {
  server.handleClient();
}